@@ -4,28 +4,32 @@ kombu.connection
 
 Broker connection and pools.
 
-:copyright: (c) 2009 - 2011 by Ask Solem.
+:copyright: (c) 2009 - 2012 by Ask Solem.
 :license: BSD, see LICENSE for more details.
 
 """
+from __future__ import absolute_import
+from __future__ import with_statement
+
 import os
 import sys
 import socket
 
+from contextlib import contextmanager
 from copy import copy
+from functools import wraps
 from itertools import count
 from Queue import Empty
 from urlparse import urlparse
 try:
     from urlparse import parse_qsl
-except ImportError:
+except ImportError:  # pragma: no cover
     from cgi import parse_qsl  # noqa
 
-from kombu import exceptions
-from kombu.transport import get_transport_cls
-from kombu.utils import kwdict, partition, retry_over_time
-from kombu.utils.compat import OrderedDict, LifoQueue as _LifoQueue
-from kombu.utils.functional import wraps
+from . import exceptions
+from .transport import get_transport_cls
+from .utils import kwdict, retry_over_time
+from .utils.compat import OrderedDict, LifoQueue as _LifoQueue
 
 _LOG_CONNECTION = os.environ.get("KOMBU_LOG_CONNECTION", False)
 _LOG_CHANNEL = os.environ.get("KOMBU_LOG_CHANNEL", False)
@@ -36,6 +40,9 @@ URI_FORMAT = """\
 %(transport)s://%(userid)s@%(hostname)s%(port)s/%(virtual_host)s\
 """
 
+__all__ = ["parse_url", "BrokerConnection", "Resource",
+           "ConnectionPool", "ChannelPool"]
+
 
 def parse_url(url):
     port = path = auth = userid = password = None
@@ -47,19 +54,19 @@ def parse_url(url):
     # use port but let pymongo get the port(s) from the URI instead.
     # This enables the use of replica sets and sharding.
     # See pymongo.Connection() for more info.
-    if scheme != 'mongodb':
+    if scheme == 'mongodb':
+        # strip the scheme since it is appended automatically.
+        hostname = url[len('mongodb://'):]
+    else:
         netloc = parts.netloc
         if '@' in netloc:
-            auth, _, netloc = partition(parts.netloc, '@')
-            userid, _, password = partition(auth, ':')
-        hostname, _, port = partition(netloc, ':')
+            auth, _, netloc = parts.netloc.partition('@')
+            userid, _, password = auth.partition(':')
+        hostname, _, port = netloc.partition(':')
         path = parts.path or ""
         if path and path[0] == '/':
             path = path[1:]
         port = port and int(port) or port
-    else:
-        # strip the scheme since it is appended automatically
-        hostname = url[len('mongodb://'):]
 
     return dict({"hostname": hostname,
                  "port": port or None,
@@ -138,8 +145,8 @@ class BrokerConnection(object):
             transport_options = {}
         self.transport_options = transport_options
 
-        if _LOG_CONNECTION:
-            from kombu.utils.log import get_logger
+        if _LOG_CONNECTION:  # pragma: no cover
+            from .log import get_logger
             self._logger = get_logger("kombu.connection")
 
     def _init_params(self, hostname, userid, password, virtual_host, port,
@@ -156,7 +163,7 @@ class BrokerConnection(object):
         self.transport_cls = transport
 
     def _debug(self, msg, ident="[Kombu connection:0x%(id)x] ", **kwargs):
-        if self._logger:
+        if self._logger:  # pragma: no cover
             self._logger.debug((ident + unicode(msg)) % {"id": id(self)},
                                **kwargs)
 
@@ -169,8 +176,8 @@ class BrokerConnection(object):
         """Request a new channel."""
         self._debug("create channel")
         chan = self.transport.create_channel(self.connection)
-        if _LOG_CHANNEL:
-            from kombu.utils.debug import Logwrapped
+        if _LOG_CHANNEL:  # pragma: no cover
+            from .utils.debug import Logwrapped
             return Logwrapped(chan, "kombu.channel",
                     "[Kombu channel:%(channel_id)s] ")
         return chan
@@ -282,7 +289,7 @@ class BrokerConnection(object):
         @wraps(fun)
         def _ensured(*args, **kwargs):
             got_connection = 0
-            for retries in count(0):
+            for retries in count(0):  # for infinity
                 try:
                     return fun(*args, **kwargs)
                 except self.connection_errors + self.channel_errors, exc:
@@ -471,13 +478,13 @@ class BrokerConnection(object):
         return ChannelPool(self, limit, preload)
 
     def Producer(self, channel=None, *args, **kwargs):
-        from kombu.messaging import Producer
+        from .messaging import Producer
         if channel is None:
             channel = self   # use default channel support.
         return Producer(channel, *args, **kwargs)
 
     def Consumer(self, queues=None, channel=None, *args, **kwargs):
-        from kombu.messaging import Consumer
+        from .messaging import Consumer
         if channel is None:
             channel = self  # use default channel support.
         return Consumer(channel, queues, *args, **kwargs)
@@ -505,7 +512,7 @@ class BrokerConnection(object):
            object.
 
         """
-        from kombu.simple import SimpleQueue
+        from .simple import SimpleQueue
 
         channel_autoclose = False
         if channel is None:
@@ -525,7 +532,7 @@ class BrokerConnection(object):
         and acknowledgements are disabled (``no_ack``).
 
         """
-        from kombu.simple import SimpleBuffer
+        from .simple import SimpleBuffer
 
         channel_autoclose = False
         if channel is None:
@@ -609,6 +616,7 @@ Connection = BrokerConnection
 
 
 class Resource(object):
+    LimitExceeded = exceptions.LimitExceeded
 
     def __init__(self, limit=None, preload=None):
         self.limit = limit
@@ -728,7 +736,7 @@ class Resource(object):
             if mutex:
                 mutex.release()
 
-    if os.environ.get("KOMBU_DEBUG_POOL"):
+    if os.environ.get("KOMBU_DEBUG_POOL"):  # pragma: no cover
         _orig_acquire = acquire
         _orig_release = release
 
@@ -755,20 +763,6 @@ class Resource(object):
             return r
 
 
-class PoolChannelContext(object):
-
-    def __init__(self, pool, block=False):
-        self.pool = pool
-        self.block = block
-
-    def __enter__(self):
-        self.conn = self.pool.acquire(block=self.block)
-        return self.conn, self.conn.default_channel
-
-    def __exit__(self, *exc_info):
-        self.conn.release()
-
-
 class ConnectionPool(Resource):
     LimitExceeded = exceptions.ConnectionLimitExceeded
 
@@ -786,8 +780,10 @@ class ConnectionPool(Resource):
     def close_resource(self, resource):
         resource._close()
 
+    @contextmanager
     def acquire_channel(self, block=False):
-        return PoolChannelContext(self, block)
+        with self.acquire(block=block) as connection:
+            yield connection, connection.default_channel
 
     def setup(self):
         if self.limit:
