@@ -4,17 +4,21 @@ kombu.entity
 
 Exchange and Queue declarations.
 
-:copyright: (c) 2009 - 2011 by Ask Solem.
+:copyright: (c) 2009 - 2012 by Ask Solem.
 :license: BSD, see LICENSE for more details.
 
 """
-from kombu.abstract import MaybeChannelBound
+from __future__ import absolute_import
+
+from .abstract import MaybeChannelBound
 
 TRANSIENT_DELIVERY_MODE = 1
 PERSISTENT_DELIVERY_MODE = 2
 DELIVERY_MODES = {"transient": TRANSIENT_DELIVERY_MODE,
                   "persistent": PERSISTENT_DELIVERY_MODE}
 
+__all__ = ["Exchange", "Queue"]
+
 
 class Exchange(MaybeChannelBound):
     """An Exchange declaration.
@@ -237,6 +241,10 @@ class Exchange(MaybeChannelBound):
         return super(Exchange, self).__repr__("Exchange %s(%s)" % (self.name,
                                                                    self.type))
 
+    @property
+    def can_cache_declaration(self):
+        return self.durable
+
 
 class Queue(MaybeChannelBound):
     """A Queue declaration.
@@ -333,7 +341,7 @@ class Queue(MaybeChannelBound):
 
     """
     name = ""
-    exchange = None
+    exchange = Exchange("")
     routing_key = ""
 
     durable = True
@@ -512,3 +520,7 @@ class Queue(MaybeChannelBound):
                  "Queue %s -> %s -> %s" % (self.name,
                                            self.exchange,
                                            self.routing_key))
+
+    @property
+    def can_cache_declaration(self):
+        return self.durable
