@@ -4,32 +4,32 @@ kombu.messaging
 
 Sending and receiving messages.
 
-:copyright: (c) 2009 - 2011 by Ask Solem.
+:copyright: (c) 2009 - 2012 by Ask Solem.
 :license: BSD, see LICENSE for more details.
 
 """
+from __future__ import absolute_import
+
 from itertools import count
 
-from kombu import entity
-from kombu.compression import compress
-from kombu.serialization import encode
-from kombu.utils import maybe_list
+from .entity import Exchange, Queue
+from .compression import compress
+from .serialization import encode
+from .utils import maybe_list
 
-
-Exchange = entity.Exchange
-Queue = entity.Queue
+__all__ = ["Exchange", "Queue", "Producer", "Consumer"]
 
 
 class Producer(object):
     """Message Producer.
 
-    :param channel: Connection channel.
-    :keyword exchange: Default exchange.
-    :keyword routing_key: Default routing key.
+    :param channel: Connection or channel.
+    :keyword exchange: Optional default exchange.
+    :keyword routing_key: Optional default routing key.
     :keyword serializer: Default serializer. Default is `"json"`.
     :keyword compression: Default compression method. Default is no
         compression.
-    :keyword auto_declare: Automatically declare the exchange
+    :keyword auto_declare: Automatically declare the default exchange
       at instantiation. Default is :const:`True`.
     :keyword on_return: Callback to call for undeliverable messages,
         when the `mandatory` or `immediate` arguments to
@@ -64,11 +64,10 @@ class Producer(object):
     def __init__(self, channel, exchange=None, routing_key=None,
             serializer=None, auto_declare=None, compression=None,
             on_return=None):
-        from kombu.connection import BrokerConnection
+        from .connection import BrokerConnection
         if isinstance(channel, BrokerConnection):
             channel = channel.default_channel
         self.channel = channel
-        self.connection = self.channel.connection.client
 
         self.exchange = exchange or self.exchange
         if self.exchange is None:
@@ -83,7 +82,6 @@ class Producer(object):
         self.exchange = self.exchange(self.channel)
         if self.auto_declare:
             self.declare()
-
         if self.on_return:
             self.channel.events["basic_return"].append(self.on_return)
 
@@ -97,10 +95,16 @@ class Producer(object):
         if self.exchange.name:
             self.exchange.declare()
 
+    def maybe_declare(self, entity, retry=False, **retry_policy):
+        if entity:
+            from .common import maybe_declare
+            return maybe_declare(entity, self.channel, retry, **retry_policy)
+
     def publish(self, body, routing_key=None, delivery_mode=None,
             mandatory=False, immediate=False, priority=0, content_type=None,
             content_encoding=None, serializer=None, headers=None,
-            compression=None, exchange=None, **properties):
+            compression=None, exchange=None, retry=False, retry_policy=None,
+            declare=[], **properties):
         """Publish message to the specified exchange.
 
         :param body: Message body.
@@ -119,6 +123,7 @@ class Producer(object):
         :keyword properties: Additional properties, see the AMQP spec.
 
         """
+        retry_policy = {} if retry_policy is None else retry_policy
         headers = headers or {}
         if routing_key is None:
             routing_key = self.routing_key
@@ -128,6 +133,10 @@ class Producer(object):
         if isinstance(exchange, Exchange):
             exchange = exchange.name
 
+        # Additional  entities to declare before publishing the message.
+        for entity in declare:
+            self.maybe_declare(entity, retry, **retry_policy)
+
         body, content_type, content_encoding = self._prepare(
                 body, serializer, content_type, content_encoding,
                 compression, headers)
@@ -138,13 +147,22 @@ class Producer(object):
                                         content_encoding,
                                         headers=headers,
                                         properties=properties)
+        publish = self._publish
+        if retry:
+            publish = self.connection.ensure(self, self._publish,
+                                             **retry_policy)
+        return publish(message, routing_key, mandatory, immediate, exchange)
+
+    def _publish(self, message, routing_key, mandatory, immediate, exchange):
         return self.exchange.publish(message, routing_key, mandatory,
                                      immediate, exchange=exchange)
 
     def revive(self, channel):
         """Revive the producer after connection loss."""
+        from .connection import BrokerConnection
+        if isinstance(channel, BrokerConnection):
+            channel = channel.default_channel
         self.channel = channel
-        self.connection = self.channel.connection.client
         self.exchange.revive(channel)
 
     def __enter__(self):
@@ -184,6 +202,13 @@ class Producer(object):
 
         return body, content_type, content_encoding
 
+    @property
+    def connection(self):
+        try:
+            return self.channel.connection.client
+        except AttributeError:
+            pass
+
 
 class Consumer(object):
     """Message consumer.
@@ -196,7 +221,7 @@ class Consumer(object):
     :keyword on_decode_error: see :attr:`on_decode_error`.
 
     """
-    #: The connection channel to use.
+    #: The connection/channel to use for this consumer.
     channel = None
 
     #: A single :class:`~kombu.entity.Queue`, or a list of queues to
@@ -228,15 +253,14 @@ class Consumer(object):
 
     _next_tag = count(1).next   # global
 
-    def __init__(self, channel, queues, no_ack=None, auto_declare=None,
+    def __init__(self, channel, queues=None, no_ack=None, auto_declare=None,
             callbacks=None, on_decode_error=None):
-        from kombu.connection import BrokerConnection
+        from .connection import BrokerConnection
         if isinstance(channel, BrokerConnection):
             channel = channel.default_channel
         self.channel = channel
-        self.connection = self.channel.connection.client
 
-        self.queues = queues
+        self.queues = [] if queues is None else queues
         if no_ack is not None:
             self.no_ack = no_ack
         if auto_declare is not None:
@@ -412,17 +436,16 @@ class Consumer(object):
           registered.
 
         """
-        if not self.callbacks:
+        callbacks = self.callbacks
+        if not callbacks:
             raise NotImplementedError("No consumer callbacks registered")
-        for callback in self.callbacks:
-            callback(body, message)
+        [callback(body, message) for callback in callbacks]
 
     def revive(self, channel):
         """Revive consumer after connection loss."""
         for queue in self.queues:
             queue.revive(channel)
         self.channel = channel
-        self.connection = self.channel.connection.client
 
     def _basic_consume(self, queue, consumer_tag=None,
             no_ack=no_ack, nowait=True):
@@ -440,10 +463,13 @@ class Consumer(object):
         self._active_tags[queue.name] = tag
         return tag
 
-    def _receive_callback(self, raw_message):
+    def _receive_callback(self, message):
+        channel = self.channel
         try:
-            message = self.channel.message_to_python(raw_message)
-            decoded = message.payload
+            m2p = getattr(channel, "message_to_python", None)
+            if m2p:
+                message = channel.message_to_python(message)
+            decoded = message.decode()
         except Exception, exc:
             if not self.on_decode_error:
                 raise
@@ -453,3 +479,10 @@ class Consumer(object):
 
     def __repr__(self):
         return "<Consumer: %s>" % (self.queues, )
+
+    @property
+    def connection(self):
+        try:
+            return self.channel.connection.client
+        except AttributeError:
+            pass
