@@ -1,16 +1,21 @@
-from kombu.tests.utils import unittest
+from __future__ import absolute_import
+from __future__ import with_statement
 
 import anyjson
 
-from kombu.connection import BrokerConnection
-from kombu.exceptions import MessageStateError
-from kombu.messaging import Consumer, Producer
-from kombu.entity import Exchange, Queue
+from mock import patch
 
-from kombu.tests.mocks import Transport
+from ..connection import BrokerConnection
+from ..exceptions import MessageStateError
+from ..messaging import Consumer, Producer
+from ..entity import Exchange, Queue
+
+from .mocks import Transport
+from .utils import TestCase
+from .utils import Mock
 
 
-class test_Producer(unittest.TestCase):
+class test_Producer(TestCase):
 
     def setUp(self):
         self.exchange = Exchange("foo", "direct")
@@ -19,6 +24,19 @@ class test_Producer(unittest.TestCase):
         self.assertTrue(self.connection.connection.connected)
         self.assertFalse(self.exchange.is_bound)
 
+    @patch("kombu.common.maybe_declare")
+    def test_maybe_declare(self, maybe_declare):
+        p = self.connection.Producer()
+        q = Queue("foo")
+        p.maybe_declare(q)
+        maybe_declare.assert_called_with(q, p.channel, False)
+
+    @patch("kombu.common.maybe_declare")
+    def test_maybe_declare_when_entity_false(self, maybe_declare):
+        p = self.connection.Producer()
+        p.maybe_declare(None)
+        self.assertFalse(maybe_declare.called)
+
     def test_auto_declare(self):
         channel = self.connection.channel()
         p = Producer(channel, self.exchange, auto_declare=True)
@@ -43,7 +61,7 @@ class test_Producer(unittest.TestCase):
         channel = self.connection.channel()
         p = Producer(channel, self.exchange, serializer="json")
         m, ctype, cencoding = p._prepare(message, headers={})
-        self.assertDictEqual(message, anyjson.deserialize(m))
+        self.assertDictEqual(message, anyjson.loads(m))
         self.assertEqual(ctype, "application/json")
         self.assertEqual(cencoding, "utf-8")
 
@@ -58,7 +76,7 @@ class test_Producer(unittest.TestCase):
         self.assertEqual(cencoding, "utf-8")
         self.assertEqual(headers["compression"], "application/x-gzip")
         import zlib
-        self.assertEqual(anyjson.deserialize(
+        self.assertEqual(anyjson.loads(
                             zlib.decompress(m).decode("utf-8")), message)
 
     def test_prepare_custom_content_type(self):
@@ -89,6 +107,45 @@ class test_Producer(unittest.TestCase):
         self.assertEqual(ctype, "text/plain")
         self.assertEqual(cencoding, "utf-8")
 
+    def test_publish_with_Exchange_instance(self):
+        p = self.connection.Producer()
+        p._publish = Mock()
+        p.publish("hello", exchange=Exchange("foo"))
+        self.assertEqual(p._publish.call_args[0][4], "foo")
+
+    def test_publish_retry_with_declare(self):
+        p = self.connection.Producer()
+        p.maybe_declare = Mock()
+        ensure = p.connection.ensure = Mock()
+        ex = Exchange("foo")
+        p.publish("hello", exchange=ex, declare=[ex], retry=True,
+                retry_policy={"step": 4})
+        p.maybe_declare.assert_called_with(ex, True, step=4)
+        ensure.assert_called_with(p, p._publish, step=4)
+
+    def test_revive_when_channel_is_connection(self):
+        p = self.connection.Producer()
+        p.exchange = Mock()
+        new_conn = BrokerConnection("memory://")
+        defchan = new_conn.default_channel
+        p.revive(new_conn)
+
+        self.assertIs(p.channel, defchan)
+        p.exchange.revive.assert_called_with(defchan)
+
+    def test_enter_exit(self):
+        p = self.connection.Producer()
+        p.release = Mock()
+
+        self.assertIs(p.__enter__(), p)
+        p.__exit__()
+        p.release.assert_called_with()
+
+    def test_connection_property_handles_AttributeError(self):
+        p = self.connection.Producer()
+        p.channel = object()
+        self.assertIsNone(p.connection)
+
     def test_publish(self):
         channel = self.connection.channel()
         p = Producer(channel, self.exchange, serializer="json")
@@ -98,7 +155,7 @@ class test_Producer(unittest.TestCase):
         self.assertIn("basic_publish", channel)
 
         m, exc, rkey = ret
-        self.assertDictEqual(message, anyjson.deserialize(m["body"]))
+        self.assertDictEqual(message, anyjson.loads(m["body"]))
         self.assertDictContainsSubset({"content_type": "application/json",
                                        "content_encoding": "utf-8",
                                        "priority": 0}, m)
@@ -130,7 +187,7 @@ class test_Producer(unittest.TestCase):
         self.assertTrue(p.on_return)
 
 
-class test_Consumer(unittest.TestCase):
+class test_Consumer(TestCase):
 
     def setUp(self):
         self.connection = BrokerConnection(transport=Transport)
@@ -144,6 +201,50 @@ class test_Consumer(unittest.TestCase):
         consumer = Consumer(channel, queue, auto_declare=True, no_ack=True)
         self.assertTrue(consumer.no_ack)
 
+    def test_add_queue_when_auto_declare(self):
+        consumer = self.connection.Consumer(auto_declare=True)
+        q = Mock()
+        q.return_value = q
+        consumer.add_queue(q)
+        self.assertIn(q, consumer.queues)
+        q.declare.assert_called_with()
+
+    def test_add_queue_when_not_auto_declare(self):
+        consumer = self.connection.Consumer(auto_declare=False)
+        q = Mock()
+        q.return_value = q
+        consumer.add_queue(q)
+        self.assertIn(q, consumer.queues)
+        self.assertFalse(q.declare.call_count)
+
+    def test_consume_without_queues_returns(self):
+        consumer = self.connection.Consumer()
+        consumer.queues[:] = []
+        self.assertIsNone(consumer.consume())
+
+    def test_consuming_from(self):
+        consumer = self.connection.Consumer()
+        consumer.queues[:] = [Queue("a"), Queue("b")]
+        self.assertFalse(consumer.consuming_from(Queue("c")))
+        self.assertFalse(consumer.consuming_from("c"))
+        self.assertTrue(consumer.consuming_from(Queue("a")))
+        self.assertTrue(consumer.consuming_from(Queue("b")))
+        self.assertTrue(consumer.consuming_from("b"))
+
+    def test_receive_callback_without_m2p(self):
+        channel = self.connection.channel()
+        c = channel.Consumer()
+        m2p = getattr(channel, "message_to_python")
+        channel.message_to_python = None
+        try:
+            message = Mock()
+            message.decode.return_value = "Hello"
+            recv = c.receive = Mock()
+            c._receive_callback(message)
+            recv.assert_called_with("Hello", message)
+        finally:
+            channel.message_to_python = m2p
+
     def test_set_callbacks(self):
         channel = self.connection.channel()
         queue = Queue("qname", self.exchange, "rkey")
@@ -290,8 +391,8 @@ class test_Consumer(unittest.TestCase):
             message.ack()
 
         consumer.register_callback(callback)
-        self.assertRaises(MessageStateError,
-                          consumer._receive_callback, {"foo": "bar"})
+        with self.assertRaises(MessageStateError):
+            consumer._receive_callback({"foo": "bar"})
 
     def test_basic_reject(self):
         channel = self.connection.channel()
@@ -315,8 +416,8 @@ class test_Consumer(unittest.TestCase):
             message.reject()
 
         consumer.register_callback(callback)
-        self.assertRaises(MessageStateError,
-                          consumer._receive_callback, {"foo": "bar"})
+        with self.assertRaises(MessageStateError):
+            consumer._receive_callback({"foo": "bar"})
         self.assertIn("basic_reject", channel)
 
     def test_basic_reject__requeue(self):
@@ -341,15 +442,16 @@ class test_Consumer(unittest.TestCase):
             message.requeue()
 
         consumer.register_callback(callback)
-        self.assertRaises(MessageStateError,
-                          consumer._receive_callback, {"foo": "bar"})
+        with self.assertRaises(MessageStateError):
+            consumer._receive_callback({"foo": "bar"})
         self.assertIn("basic_reject:requeue", channel)
 
     def test_receive_without_callbacks_raises(self):
         channel = self.connection.channel()
         b1 = Queue("qname1", self.exchange, "rkey")
         consumer = Consumer(channel, [b1])
-        self.assertRaises(NotImplementedError, consumer.receive, 1, 2)
+        with self.assertRaises(NotImplementedError):
+            consumer.receive(1, 2)
 
     def test_decode_error(self):
         channel = self.connection.channel()
@@ -357,8 +459,8 @@ class test_Consumer(unittest.TestCase):
         consumer = Consumer(channel, [b1])
         consumer.channel.throw_decode_error = True
 
-        self.assertRaises(ValueError,
-                consumer._receive_callback, {"foo": "bar"})
+        with self.assertRaises(ValueError):
+            consumer._receive_callback({"foo": "bar"})
 
     def test_on_decode_error_callback(self):
         channel = self.connection.channel()
@@ -374,7 +476,7 @@ class test_Consumer(unittest.TestCase):
 
         self.assertTrue(thrown)
         m, exc = thrown[0]
-        self.assertEqual(anyjson.deserialize(m), {"foo": "bar"})
+        self.assertEqual(anyjson.loads(m), {"foo": "bar"})
         self.assertIsInstance(exc, ValueError)
 
     def test_recover(self):
@@ -398,3 +500,8 @@ class test_Consumer(unittest.TestCase):
         channel = self.connection.channel()
         b1 = Queue("qname1", self.exchange, "rkey")
         self.assertTrue(repr(Consumer(channel, [b1])))
+
+    def test_connection_property_handles_AttributeError(self):
+        p = self.connection.Consumer()
+        p.channel = object()
+        self.assertIsNone(p.connection)
