@@ -6,10 +6,12 @@ Virtual transport implementation.
 
 Emulates the AMQ API for non-AMQ transports.
 
-:copyright: (c) 2009, 2011 by Ask Solem.
+:copyright: (c) 2009, 2012 by Ask Solem.
 :license: BSD, see LICENSE for more details.
 
 """
+from __future__ import absolute_import
+
 import base64
 import socket
 import warnings
@@ -18,15 +20,16 @@ from itertools import count
 from time import sleep, time
 from Queue import Empty
 
-from kombu.exceptions import StdChannelError
-from kombu.transport import base
-from kombu.utils import emergency_dump_state, say
-from kombu.utils.compat import OrderedDict
-from kombu.utils.encoding import str_to_bytes, bytes_to_str
-from kombu.utils.finalize import Finalize
+from ...exceptions import StdChannelError
+from ...utils import emergency_dump_state, say
+from ...utils.compat import OrderedDict
+from ...utils.encoding import str_to_bytes, bytes_to_str
+from ...utils.finalize import Finalize
 
-from kombu.transport.virtual.scheduling import FairCycle
-from kombu.transport.virtual.exchange import STANDARD_EXCHANGE_TYPES
+from .. import base
+
+from .scheduling import FairCycle
+from .exchange import STANDARD_EXCHANGE_TYPES
 
 UNDELIVERABLE_FMT = """\
 Message could not be delivered: No queues bound to exchange %(exchange)r
@@ -157,7 +160,7 @@ class QoS(object):
 
             try:
                 self.channel._restore(message)
-            except (KeyboardInterrupt, SystemExit, Exception), exc:
+            except BaseException, exc:
                 errors.append((exc, message))
         delivered.clear()
         return errors
@@ -172,7 +175,7 @@ class QoS(object):
         self._flush()
         state = self._delivered
 
-        if not self.channel.do_restore or getattr(state, "restored"):
+        if not self.channel.do_restore or getattr(state, "restored", None):
             assert not state
             return
 
@@ -392,7 +395,7 @@ class Channel(AbstractChannel, base.StdChannel):
     def after_reply_message_received(self, queue):
         self.queue_delete(queue)
 
-    def queue_bind(self, queue, exchange, routing_key, arguments=None,
+    def queue_bind(self, queue, exchange, routing_key="", arguments=None,
             **kwargs):
         """Bind `queue` to `exchange` with `routing key`."""
         if queue in self.state.bindings:
