@@ -4,10 +4,12 @@ kombu.utils.eventio
 
 Evented IO support for multiple platforms.
 
-:copyright: (c) 2009 - 2011 by Ask Solem.
+:copyright: (c) 2009 - 2012 by Ask Solem.
 :license: BSD, see LICENSE for more details.
 
 """
+from __future__ import absolute_import
+
 import errno
 import select
 import socket
@@ -17,6 +19,8 @@ try:
 except ImportError:
     is_eventlet = lambda module: False  # noqa
 
+__all__ = ["poll"]
+
 POLL_READ = 0x001
 POLL_ERR = 0x008 | 0x010 | 0x2000
 
@@ -59,7 +63,7 @@ class _epoll(Poller):
             pass
 
     def _poll(self, timeout):
-        return self._epoll.poll(timeout and timeout / 1000.0 or -1)
+        return self._epoll.poll(timeout or -1)
 
 
 class _kqueue(Poller):
@@ -84,8 +88,7 @@ class _kqueue(Poller):
                                                 flags=flags)], 0)
 
     def _poll(self, timeout):
-        kevents = self._kqueue.control(None, 1000,
-                                      timeout and timeout / 1000.0 or timeout)
+        kevents = self._kqueue.control(None, 1000, timeout)
         events = {}
         for kevent in kevents:
             fd = kevent.ident
